from typing import List, Union

import pytest

from gcs_migration.typing.gcs_documents import IdentityMappingDict
from gcs_migration.models import (
    BlackPearlStorageGateway,
    CephStorageGateway,
    HPSSStorageGateway,
    StorageGateway,
    S3StorageGateway,
)

from gcs_migration.check.storage_gateway import (
    StorageGatewayCheck,
    SG01,
    SG02,
    SG03,
    SG04,
    SG05,
    SG06,
    SG07,
    SG08,
    SG09,
    SG10,
    SG11,
    SG12,
    SG13,
    SG14,
    SG15,
    SG16,
    SG17,
    SG18,
    SG19,
    SG20,
    SG21,
    SG22,
    SG23,
    SG24,
)


def test_check_sg01(storage_gateway: StorageGateway) -> None:
    sg01 = SG01()

    assert not sg01.is_ok(None)
    assert sg01.is_ok(storage_gateway)

    checker = StorageGatewayCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, SG01), res)) != []

    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG01), res)) == []


def test_check_sg02(storage_gateway: StorageGateway) -> None:
    sg02 = SG02()

    storage_gateway.display_name = None

    assert sg02.is_ok(None)
    assert not sg02.is_ok(storage_gateway)

    checker = StorageGatewayCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, SG02), res)) == []

    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG02), res)) != []

    storage_gateway.display_name = "OK"
    assert sg02.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG02), res)) == []

    storage_gateway.display_name = "\n"
    assert sg02.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG02), res)) == []


def test_check_sg03(storage_gateway: StorageGateway) -> None:
    sg03 = SG03()
    checker = StorageGatewayCheck()

    storage_gateway.display_name = "OK"

    assert sg03.is_ok(None)
    assert sg03.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG03), res)) == []

    storage_gateway.display_name = "\n"
    assert not sg03.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG03), res)) != []


def test_check_sg04(storage_gateway: StorageGateway) -> None:
    sg04 = SG04()
    checker = StorageGatewayCheck()

    storage_gateway.allowed_domains = ["domain"]

    assert sg04.is_ok(None)
    assert sg04.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG04), res)) == []

    storage_gateway.allowed_domains = None
    assert not sg04.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG04), res)) != []


def test_check_sg05(storage_gateway: StorageGateway) -> None:
    sg05 = SG05()
    checker = StorageGatewayCheck()

    storage_gateway.allowed_domains = ["domain"]

    assert sg05.is_ok(None)
    assert sg05.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG05), res)) == []

    storage_gateway.allowed_domains = []
    assert not sg05.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG05), res)) != []


@pytest.mark.parametrize(
    "domains,use_mapping,expect",
    [
        (None, False, True),
        (None, True, True),
        (["example.edu"], False, True),
        (["example.edu"], True, True),
        (["example.edu", "example.org"], False, False),
        (["example.edu", "example.org"], True, True),
    ],
)
def test_check_sg06(
    storage_gateway: StorageGateway,
    storage_gateway_identity_mapping: List[IdentityMappingDict],
    domains: Union[None, List[str]],
    use_mapping: bool,
    expect: bool,
) -> None:
    sg06 = SG06()
    checker = StorageGatewayCheck()

    storage_gateway.allowed_domains = domains
    if use_mapping:
        storage_gateway.identity_mappings = storage_gateway_identity_mapping
    else:
        storage_gateway.identity_mappings = None

    assert sg06.is_ok(None)

    assert sg06.is_ok(storage_gateway) == expect
    res = checker.check(storage_gateway)

    assert (list(filter(lambda x: isinstance(x, SG06), res)) == []) == expect


def test_check_sg07(
    storage_gateway: StorageGateway,
) -> None:
    sg07 = SG07()
    checker = StorageGatewayCheck()

    storage_gateway.load_dsi_module = None

    assert sg07.is_ok(None)
    assert sg07.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG07), res)) == []

    storage_gateway.load_dsi_module = "google_drive"
    assert not sg07.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG07), res)) != []


def test_check_sg08(
    connector: str,
    storage_gateway: StorageGateway,
) -> None:
    sg08 = SG08()
    checker = StorageGatewayCheck()

    assert sg08.is_ok(None)
    assert sg08.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG08), res)) == []

    if connector == "s3":
        assert isinstance(storage_gateway, S3StorageGateway)
        storage_gateway.s3_endpoint = None
        assert not sg08.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG08), res)) != []


def test_check_sg09(
    connector: str,
    storage_gateway: StorageGateway,
) -> None:
    sg09 = SG09()
    checker = StorageGatewayCheck()

    assert sg09.is_ok(None)
    assert sg09.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG09), res)) == []

    if connector == "s3":
        assert isinstance(storage_gateway, S3StorageGateway)
        storage_gateway.s3_user_credential_required = None
        assert not sg09.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG09), res)) != []


def test_check_sg10(
    connector: str,
    storage_gateway: StorageGateway,
) -> None:
    sg10 = SG10()
    checker = StorageGatewayCheck()

    assert sg10.is_ok(None)
    assert sg10.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG10), res)) == []

    if connector == "blackpearl":
        assert isinstance(storage_gateway, BlackPearlStorageGateway)
        storage_gateway.bp_access_id_file = None
        assert not sg10.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG10), res)) != []


def test_check_sg11(
    connector: str,
    storage_gateway: StorageGateway,
    blackpearl_access_id_file: str,
) -> None:
    sg11 = SG11()
    checker = StorageGatewayCheck()

    assert sg11.is_ok(None)
    assert sg11.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG11), res)) == []

    if connector == "blackpearl":
        assert isinstance(storage_gateway, BlackPearlStorageGateway)
        storage_gateway.bp_access_id_file = "/not-existing/file"
        assert not sg11.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG11), res)) != []


def test_check_sg12(
    connector: str,
    storage_gateway: StorageGateway,
) -> None:
    sg12 = SG12()
    checker = StorageGatewayCheck()

    assert sg12.is_ok(None)
    assert sg12.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG12), res)) == []

    if connector == "ceph":
        assert isinstance(storage_gateway, CephStorageGateway)
        storage_gateway.ceph_admin_bucket = "not_admin"
        assert not sg12.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG12), res)) != []


def test_check_sg13(
    connector: str,
    storage_gateway: StorageGateway,
) -> None:
    sg13 = SG13()
    checker = StorageGatewayCheck()

    assert sg13.is_ok(None)
    assert sg13.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG13), res)) == []

    if connector == "ceph":
        assert isinstance(storage_gateway, CephStorageGateway)
        storage_gateway.ceph_admin_key_id = None
        assert not sg13.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG13), res)) != []


def test_check_sg14(
    connector: str,
    storage_gateway: StorageGateway,
) -> None:
    sg14 = SG14()
    checker = StorageGatewayCheck()

    assert sg14.is_ok(None)
    assert sg14.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG14), res)) == []

    if connector == "ceph":
        assert isinstance(storage_gateway, CephStorageGateway)
        storage_gateway.ceph_admin_key_id = ""
        assert not sg14.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG14), res)) != []


def test_check_sg15(
    connector: str,
    storage_gateway: StorageGateway,
) -> None:
    sg15 = SG15()
    checker = StorageGatewayCheck()

    assert sg15.is_ok(None)
    assert sg15.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG15), res)) == []

    if connector == "ceph":
        assert isinstance(storage_gateway, CephStorageGateway)
        storage_gateway.ceph_admin_secret_key = None
        assert not sg15.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG15), res)) != []


def test_check_sg16(
    connector: str,
    storage_gateway: StorageGateway,
) -> None:
    sg16 = SG16()
    checker = StorageGatewayCheck()

    assert sg16.is_ok(None)
    assert sg16.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG16), res)) == []

    if connector == "ceph":
        assert isinstance(storage_gateway, CephStorageGateway)
        storage_gateway.ceph_admin_secret_key = ""
        assert not sg16.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG16), res)) != []


# MissingPropertyCheck: HPSSStoageGateway.login_name is not None
def test_check_sg17(
    connector: str,
    storage_gateway: StorageGateway,
):
    sg17 = SG17()
    checker = StorageGatewayCheck()

    assert sg17.is_ok(None)
    assert sg17.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG17), res)) == []

    if connector == "hpss_local":
        assert isinstance(storage_gateway, HPSSStorageGateway)
        storage_gateway.login_name = None
        assert not sg17.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG17), res)) != []


# PropertyConstraintCheck: Warn if HPSSStorageGateway.login_name != 'hpssftp'
def test_check_sg18(
    connector: str,
    storage_gateway: StorageGateway,
):
    sg18 = SG18()
    checker = StorageGatewayCheck()

    assert sg18.is_ok(None)
    assert sg18.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG18), res)) == []

    if connector == "hpss_local":
        assert isinstance(storage_gateway, HPSSStorageGateway)
        storage_gateway.login_name = "foobar"
        assert sg18.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG18), res)) == []


# MissingPropertyCheck: HPSSStorageGateway.authentication_mech is not None
def test_check_sg19(
    connector: str,
    storage_gateway: StorageGateway,
):
    sg19 = SG19()
    checker = StorageGatewayCheck()

    assert sg19.is_ok(None)
    assert sg19.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG19), res)) == []

    if connector == "hpss_local":
        assert isinstance(storage_gateway, HPSSStorageGateway)
        storage_gateway.authentication_mech = None
        assert not sg19.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG19), res)) != []


# PropertyConstraintCheck: authentication_mech must be 'unix' or 'krb5'
def test_check_sg20(
    connector: str,
    storage_gateway: StorageGateway,
):
    sg20 = SG20()
    checker = StorageGatewayCheck()

    assert sg20.is_ok(None)
    assert sg20.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG20), res)) == []

    if connector == "hpss_local":
        assert isinstance(storage_gateway, HPSSStorageGateway)
        storage_gateway.authentication_mech = "foobar"
        assert not sg20.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG20), res)) != []

        storage_gateway.authentication_mech = "unix"
        assert sg20.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG20), res)) == []

        storage_gateway.authentication_mech = "krb5"
        assert sg20.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG20), res)) == []


# MissingPropertyCheck: HPSSStorageGateway.authenticator is not None
def test_check_sg21(
    connector: str,
    storage_gateway: StorageGateway,
):
    sg21 = SG21()
    checker = StorageGatewayCheck()

    assert sg21.is_ok(None)
    assert sg21.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG21), res)) == []

    if connector == "hpss_local":
        assert isinstance(storage_gateway, HPSSStorageGateway)

        storage_gateway.authenticator = None
        assert not sg21.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG21), res)) != []


# PropertyConstraintCheck: HPSSStorageGateway.authenticator has correct format
def test_check_sg22(
    connector: str,
    storage_gateway: StorageGateway,
):
    sg22 = SG22()
    checker = StorageGatewayCheck()

    assert sg22.is_ok(None)
    assert sg22.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG22), res)) == []

    if connector == "hpss_local":
        assert isinstance(storage_gateway, HPSSStorageGateway)
        storage_gateway.authenticator = "auth_keyfile:"
        assert not sg22.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG22), res)) != []

        storage_gateway.authenticator = "auth_keytab:"
        assert not sg22.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG22), res)) != []

        storage_gateway.authenticator = "foo:"
        assert not sg22.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG22), res)) != []


# PropertyConstraintCheck: HPSSStorageGateway.authenticator file must exist
def test_check_sg23(
    connector: str,
    storage_gateway: StorageGateway,
    hpss_config_file,
):
    sg23 = SG23()
    checker = StorageGatewayCheck()

    assert sg23.is_ok(None)
    assert sg23.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG23), res)) == []

    if connector == "hpss_local":
        assert isinstance(storage_gateway, HPSSStorageGateway)
        storage_gateway.authenticator = "auth_keyfile:/not-existing/file"
        assert not sg23.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG23), res)) != []


# MissingPropertyCheck: HPSSStorageGateway.uda_checksum is not None
def test_check_sg24(
    connector: str,
    storage_gateway: StorageGateway,
):
    sg24 = SG24()
    checker = StorageGatewayCheck()

    assert sg24.is_ok(None)
    assert sg24.is_ok(storage_gateway)
    res = checker.check(storage_gateway)
    assert list(filter(lambda x: isinstance(x, SG24), res)) == []

    if connector == "hpss_local":
        assert isinstance(storage_gateway, HPSSStorageGateway)
        storage_gateway.uda_checksum = None
        assert not sg24.is_ok(storage_gateway)
        res = checker.check(storage_gateway)
        assert list(filter(lambda x: isinstance(x, SG24), res)) != []
