import json
import os
from typing import Optional
from zipfile import ZipFile

import pytest
from pytest_lazyfixture import lazy_fixture
from pyfakefs.fake_filesystem import FakeFilesystem
from click.testing import CliRunner

from gcs_migration.commands.gcs_migration import main
from gcs_migration.models import (
    BlackPearlCredential,
    CephStorageGateway,
    MigrationPlan,
    S3Credential,
)
from gcs_migration.util import yaml


@pytest.mark.parametrize("description", [None, "This is a snapshot"])
def test_snapshot_create(
    migration_plan: MigrationPlan,
    fs: FakeFilesystem,
    description: Optional[str],
) -> None:
    migration_plan.dump()

    runner = CliRunner()
    args = ["snapshot", "create"]

    if description is not None:
        args.extend(["-d", description])

    os.mkdir("migration_snapshots")
    with open("migration_snapshots/0009", "w"):
        pass

    result = runner.invoke(main, args, catch_exceptions=False)
    assert result.exit_code == 0

    assert os.path.exists("migration_snapshots/0010")

    with ZipFile("migration_snapshots/0010", "r") as z:
        with z.open("metadata.json", "r") as mf:
            metadata = json.load(mf)
            assert metadata.get("description") == description
            for n in z.namelist():
                if n != "metadata.json":
                    assert os.path.exists(n)


@pytest.mark.parametrize("omit_secrets", [False, True])
@pytest.mark.parametrize(
    "migration_plan_with_secrets",
    [
        lazy_fixture("migration_plan_s3"),
        lazy_fixture("migration_plan_blackpearl"),
        lazy_fixture("migration_plan_ceph"),
    ],
)
def test_snapshot_create_omit_secrets(
    migration_plan_with_secrets: MigrationPlan,
    fs: FakeFilesystem,
    omit_secrets: bool,
) -> None:
    migration_plan_with_secrets.dump()

    runner = CliRunner()
    args = ["snapshot", "create"]

    if omit_secrets:
        args.extend(["-o"])

    os.mkdir("migration_snapshots")
    with open("migration_snapshots/0009", "w"):
        pass

    result = runner.invoke(main, args, catch_exceptions=False)
    assert result.exit_code == 0

    assert os.path.exists("migration_snapshots/0010")

    with ZipFile("migration_snapshots/0010", "r") as z:
        with z.open("metadata.json", "r") as mf:
            metadata = json.load(mf)
            assert metadata.get("omit_secrets", False) == omit_secrets
            for n in z.namelist():
                if n != "metadata.json":
                    assert os.path.exists(n)
            for orig_cred in migration_plan_with_secrets.credentials:
                if isinstance(orig_cred, S3Credential):
                    with z.open(
                        f"migration_plan/credentials/{orig_cred.id}.yaml", "r"
                    ) as cred_stream:
                        new_cred = yaml.load(cred_stream)
                        assert isinstance(new_cred, S3Credential)
                        assert new_cred.s3_key_id == orig_cred.s3_key_id
                        if omit_secrets:
                            assert new_cred.s3_secret_key is None
                        else:
                            assert new_cred.s3_secret_key is not None
                            assert new_cred.s3_secret_key == orig_cred.s3_secret_key
                elif isinstance(orig_cred, BlackPearlCredential):
                    with z.open(
                        f"migration_plan/credentials/{orig_cred.id}.yaml", "r"
                    ) as cred_stream:
                        new_cred = yaml.load(cred_stream)
                        assert isinstance(new_cred, BlackPearlCredential)
                        assert new_cred.access_id == orig_cred.access_id
                        if omit_secrets:
                            assert new_cred.secret_key is None
                        else:
                            assert new_cred.secret_key is not None
                            assert new_cred.secret_key == orig_cred.secret_key
            orig_storage_gateway = migration_plan_with_secrets.storage_gateway
            if isinstance(orig_storage_gateway, CephStorageGateway):
                with z.open(
                    "migration_plan/storage_gateway.yaml", "r"
                ) as storage_gateway_stream:
                    new_storage_gateway = yaml.load(storage_gateway_stream)
                    assert isinstance(new_storage_gateway, CephStorageGateway)
                    assert (
                        new_storage_gateway.ceph_admin_key_id
                        == orig_storage_gateway.ceph_admin_key_id
                    )
                    if omit_secrets:
                        assert new_storage_gateway.ceph_admin_secret_key is None
                    else:
                        assert new_storage_gateway.ceph_admin_secret_key is not None
                        assert (
                            new_storage_gateway.ceph_admin_secret_key
                            == orig_storage_gateway.ceph_admin_secret_key
                        )
